@@ -1,8 +1,10 @@
 """Implementation of var-naming rule."""
+from __future__ import annotations
+
 import keyword
 import re
 import sys
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union
+from typing import TYPE_CHECKING, Any
 
 from ansible.parsing.yaml.objects import AnsibleUnicode
 
@@ -29,13 +31,13 @@ FAIL_PLAY = """---
     CamelCaseButErrorIgnored: true  # noqa: var-naming
 
   tasks:
-    - name: foo
+    - name: Foo
       ansible.builtin.set_fact:
         "{{ 'test_' }}var": "value"  # valid
-    - name: bar
+    - name: Bar
       ansible.builtin.set_fact:
         CamelCaseButErrorIgnored: true  # noqa: var-naming
-    - name: test in a block
+    - name: Test in a block
       block:
         - name:
           ansible.builtin.set_fact:
@@ -98,13 +100,13 @@ class VariableNamingRule(AnsibleLintRule):
         # safety measure.
         return not bool(self.re_pattern.match(ident))
 
-    def matchplay(
-        self, file: "Lintable", data: "odict[str, Any]"
-    ) -> List["MatchError"]:
+    def matchplay(self, file: Lintable, data: odict[str, Any]) -> list[MatchError]:
         """Return matches found for a specific playbook."""
-        results: List["MatchError"] = []
-        raw_results: List["MatchError"] = []
+        results: list[MatchError] = []
+        raw_results: list[MatchError] = []
 
+        if not data:
+            return results
         # If the Play uses the 'vars' section to set variables
         our_vars = data.get("vars", {})
         for key in our_vars.keys():
@@ -131,8 +133,8 @@ class VariableNamingRule(AnsibleLintRule):
         return results
 
     def matchtask(
-        self, task: Dict[str, Any], file: Optional[Lintable] = None
-    ) -> Union[bool, str]:
+        self, task: dict[str, Any], file: Lintable | None = None
+    ) -> bool | str:
         """Return matches for task based variables."""
         # If the task uses the 'vars' section to set variables
         our_vars = task.get("vars", {})
@@ -155,13 +157,13 @@ class VariableNamingRule(AnsibleLintRule):
 
         return False
 
-    def matchyaml(self, file: Lintable) -> List["MatchError"]:
+    def matchyaml(self, file: Lintable) -> list[MatchError]:
         """Return matches for variables defined in vars files."""
-        results: List["MatchError"] = []
-        raw_results: List["MatchError"] = []
-        meta_data: Dict[AnsibleUnicode, Any] = {}
+        results: list[MatchError] = []
+        raw_results: list[MatchError] = []
+        meta_data: dict[AnsibleUnicode, Any] = {}
 
-        if str(file.kind) == "vars":
+        if str(file.kind) == "vars" and file.data:
             meta_data = parse_yaml_from_file(str(file.path))
             for key in meta_data.keys():
                 if self.is_invalid_variable_name(key):
